import { bioEntityContentSchema } from '@/models/bioEntityContentSchema';
import { submapConnection } from '@/models/submapConnection';
import { apiPath } from '@/redux/apiPath';
import { axiosInstance, axiosInstanceNewAPI } from '@/services/api/utils/axiosInstance';
import { BioEntityContent, BioEntityResponse, SubmapConnection } from '@/types/models';
import { getErrorMessage } from '@/utils/getErrorMessage';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { z } from 'zod';
import { MULTI_BIO_ENTITY_FETCHING_ERROR_PREFIX } from '../bioEntity.constants';

export const getSubmapConnectionsBioEntity = createAsyncThunk<BioEntityContent[]>(
  'project/getSubmapConnectionsBioEntity',
  // eslint-disable-next-line consistent-return
  async (_, { rejectWithValue }) => {
    try {
      const response = await axiosInstance.get<SubmapConnection[]>(apiPath.getSubmapConnections());

      const isDataValid = validateDataUsingZodSchema(response.data, z.array(submapConnection));
      if (!isDataValid) {
        throw new Error('Submap connections validation error');
      }

      const searchQueries = response.data.map(({ from }) => `${from.id}`);

      const asyncFetchBioEntityFunctions = searchQueries.map(searchQuery =>
        axiosInstanceNewAPI.get<BioEntityResponse>(
          apiPath.getBioEntityContentsStringWithQuery({ searchQuery, isPerfectMatch: true }),
        ),
      );

      const bioEntityContentResponse = await Promise.all(asyncFetchBioEntityFunctions);

      const bioEntityContents = bioEntityContentResponse
        .map(contentResponse => contentResponse?.data?.content || [])
        .flat()
        .filter(content => bioEntityContentSchema.safeParse(content).success)
        .filter(payload => 'bioEntity' in payload || {});

      return bioEntityContents;
    } catch (error) {
      const errorMessage = getErrorMessage({
        error,
        prefix: MULTI_BIO_ENTITY_FETCHING_ERROR_PREFIX,
      });

      return rejectWithValue(errorMessage);
    }
  },
);
