import { createAsyncThunk } from '@reduxjs/toolkit';
import { getErrorMessage } from '@/utils/getErrorMessage';
import { ThunkConfig } from '@/types/store';
import { ConstantType } from '@/redux/constant/constant.types';
import { getConfigValue } from '@/constants/index.utils';

export const getConstant = createAsyncThunk<ConstantType | undefined, void, ThunkConfig>(
  'constant/getConstant',
  async (_, { rejectWithValue }) => {
    try {
      const apiBaseUrl = getConfigValue('BASE_API_URL');
      const result: ConstantType = {
        apiBaseUrl,
        apiDocsUrl: `${apiBaseUrl}/../docs/`,
      };
      return result;
    } catch (error) {
      const errorMessage = getErrorMessage({ error, prefix: 'Failed to build constants' });
      return rejectWithValue(errorMessage);
    }
  },
);
