/* eslint-disable no-magic-numbers */
import { axiosInstanceNewAPI } from '@/services/api/utils/axiosInstance';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { PROJECT_ID } from '@/constants';
import { ExportNetwork, ExportElements } from '@/types/models';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { exportNetworkchema, exportElementsSchema } from '@/models/exportSchema';
import { getErrorMessage } from '@/utils/getErrorMessage';
import { ThunkConfig } from '@/types/store';
import { apiPath } from '../apiPath';
import { downloadFileFromBlob } from './export.utils';
import { ELEMENTS_DOWNLOAD_ERROR_PREFIX, NETWORK_DOWNLOAD_ERROR_PREFIX } from './export.constants';

type DownloadElementsBodyRequest = {
  columns: string[];
  submaps: number[];
  annotations: string[];
  includedCompartmentIds: number[];
  excludedCompartmentIds: number[];
};

export const downloadElements = createAsyncThunk<
  undefined,
  DownloadElementsBodyRequest,
  ThunkConfig
  // eslint-disable-next-line consistent-return
>('export/downloadElements', async (data, { rejectWithValue }) => {
  try {
    const response = await axiosInstanceNewAPI.post<ExportElements>(
      apiPath.downloadElementsCsv(),
      data,
      {
        withCredentials: true,
      },
    );

    const isDataValid = validateDataUsingZodSchema(response.data, exportElementsSchema);

    if (isDataValid) {
      downloadFileFromBlob(response.data, `${PROJECT_ID}-elementExport.csv`);
    }
  } catch (error) {
    const errorMessage = getErrorMessage({ error, prefix: ELEMENTS_DOWNLOAD_ERROR_PREFIX });

    return rejectWithValue(errorMessage);
  }
});

type DownloadNetworkBodyRequest = {
  columns: string[];
  submaps: number[];
  annotations: string[];
  includedCompartmentIds: number[];
  excludedCompartmentIds: number[];
};

export const downloadNetwork = createAsyncThunk<undefined, DownloadNetworkBodyRequest, ThunkConfig>(
  'export/downloadNetwork',
  // eslint-disable-next-line consistent-return
  async (data, { rejectWithValue }) => {
    try {
      const response = await axiosInstanceNewAPI.post<ExportNetwork>(
        apiPath.downloadNetworkCsv(),
        data,
        {
          withCredentials: true,
        },
      );

      const isDataValid = validateDataUsingZodSchema(response.data, exportNetworkchema);

      if (isDataValid) {
        downloadFileFromBlob(response.data, `${PROJECT_ID}-networkExport.csv`);
      }
    } catch (error) {
      const errorMessage = getErrorMessage({ error, prefix: NETWORK_DOWNLOAD_ERROR_PREFIX });

      return rejectWithValue(errorMessage);
    }
  },
);
