import { showToast } from '@/utils/showToast';
import { errorMiddlewareListener } from './error.middleware';

jest.mock('../../utils/showToast', () => ({
  showToast: jest.fn(),
}));

describe('errorMiddlewareListener', () => {
  beforeEach(() => {
    jest.clearAllMocks();
  });

  it('should show toast with error message when action is rejected with value', async () => {
    const action = {
      type: 'action/rejected',
      payload: 'Error message',
      meta: {
        requestId: '421',
        rejectedWithValue: true,
        requestStatus: 'rejected',
      },
    };
    await errorMiddlewareListener(action);
    expect(showToast).toHaveBeenCalledWith({ type: 'error', message: 'Error message' });
  });

  it('should show toast with unknown error when action is rejected without value', async () => {
    const action = {
      type: 'action/rejected',
      payload: null,
      meta: {
        requestId: '421',
        rejectedWithValue: true,
        requestStatus: 'rejected',
      },
    };
    await errorMiddlewareListener(action);
    expect(showToast).toHaveBeenCalledWith({
      type: 'error',
      message: 'An unknown error occurred. Please try again later.',
    });
  });

  it('should not show toast when action is not rejected', async () => {
    const action = {
      type: 'action/loading',
      payload: null,
      meta: {
        requestId: '421',
        requestStatus: 'fulfilled',
      },
    };
    await errorMiddlewareListener(action);
    expect(showToast).not.toHaveBeenCalled();
  });

  it('should show toast with unknown error when action payload is not a string', async () => {
    const action = {
      type: 'action/rejected',
      payload: {},
      meta: {
        requestId: '421',
        rejectedWithValue: true,
        requestStatus: 'rejected',
      },
    };
    await errorMiddlewareListener(action);
    expect(showToast).toHaveBeenCalledWith({
      type: 'error',
      message: 'An unknown error occurred. Please try again later.',
    });
  });

  it('should show toast with custom message when action payload is a string', async () => {
    const action = {
      type: 'action/rejected',
      payload: 'Failed to fetch',
      meta: {
        requestId: '421',
        rejectedWithValue: true,
        requestStatus: 'rejected',
      },
    };
    await errorMiddlewareListener(action);
    expect(showToast).toHaveBeenCalledWith({ type: 'error', message: 'Failed to fetch' });
  });
});
