import { twMerge } from 'tailwind-merge';
import { Icon } from '../Icon';

type ToastArgs = {
  type: 'success' | 'error';
  message: string;
  onDismiss: () => void;
};

export const Toast = ({ type, message, onDismiss }: ToastArgs): React.ReactNode => (
  <div
    className={twMerge(
      'flex h-[76px] w-[700px] items-center rounded-l rounded-r-lg bg-white p-4 drop-shadow before:absolute before:inset-y-0 before:left-0 before:block before:w-1 before:rounded-l-lg before:content-[""]',
      type === 'error' ? 'before:bg-red-500' : 'before:bg-green-500',
    )}
  >
    <p
      className={twMerge(
        'text-base font-bold ',
        type === 'error' ? 'text-red-500' : 'text-green-500',
      )}
    >
      {message}
    </p>

    <button type="button" onClick={onDismiss} className="ml-auto flex-none">
      <Icon name="close" className="ml-3 h-7 w-7 fill-font-500" />
    </button>
  </div>
);
